<?php

/**
 * This is the model class for table "gportal_apalines_mm".
 *
 * The followings are the available columns in table 'gportal_apalines_mm':
 * @property integer $id
 * @property string $title
 * @property integer $ndate
 * @property string $channel
 * @property string $docid
 * @property integer $related_maindoc_id
 * @property string $place
 * @property string $source
 * @property string $filename
 * @property string $author
 * @property string $keywords
 * @property string $xml_filename
 * @property integer $deleted
 * @property integer $created
 * @property integer $updated
 * @property integer $type
 * @property string $apa_id
 */
class ApalinesMM extends APAActiveRecord
{
    const IMAGE = 0;
    const VIDEO = 1;

	/**
	 * Returns the static model of the specified AR class.
	 * @param string $className active record class name.
	 * @return ApalinesMM the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return 'gportal_apalines_mm';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('ndate, related_maindoc_id, deleted, created, updated, type', 'numerical', 'integerOnly'=>true),
			array('title, channel, docid, place, source, author, xml_filename, apa_id', 'length', 'max'=>255),
			array('filename', 'length', 'max'=>64),
			array('keywords', 'safe'),
			// The following rule is used by search().
			// Please remove those attributes that should not be searched.
			array('id, title, ndate, channel, docid, related_maindoc_id, place, source, filename, author, keywords, xml_filename, deleted, created, updated, type', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
            'files'=>array(self::HAS_MANY, 'ApalinesFile', 'parent_id'),
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'title' => 'Title',
			'ndate' => 'Ndate',
			'channel' => 'Channel',
			'docid' => 'Docid',
			'related_maindoc_id' => 'Related Maindoc',
			'place' => 'Place',
			'source' => 'Source',
			'filename' => 'Filename',
			'author' => 'Author',
			'keywords' => 'Keywords',
			'xml_filename' => 'Xml Filename',
			'deleted' => 'Deleted',
			'created' => 'Created',
			'updated' => 'Updated',
			'type' => 'Type',
            'apa_id' => 'APA id'
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
	 */
	public function search()
	{
		// Warning: Please modify the following code to remove attributes that
		// should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('title',$this->title,true);
		$criteria->compare('ndate',$this->ndate);
		$criteria->compare('channel',$this->channel,true);
		$criteria->compare('docid',$this->docid,true);
		$criteria->compare('related_maindoc_id',$this->related_maindoc_id);
		$criteria->compare('place',$this->place,true);
		$criteria->compare('source',$this->source,true);
		$criteria->compare('filename',$this->filename,true);
		$criteria->compare('author',$this->author,true);
		$criteria->compare('keywords',$this->keywords,true);
		$criteria->compare('xml_filename',$this->xml_filename,true);
		$criteria->compare('deleted',$this->deleted);
		$criteria->compare('created',$this->created);
		$criteria->compare('updated',$this->updated);
		$criteria->compare('type',$this->type);
        $criteria->compare('apa_id',$this->apa_id, true);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}
    /**
     * Function removes multimedia records and files
     */
    public function deleteFiles() {
        if (isset($this->files)) {
            foreach ($this->files as $file) {
                $file->delete();
            }
        }
    }
    /**
     * Function removes files before record is deleted
     */
    public function beforeDelete() {
        $this->deleteFiles();
        return parent::beforeDelete();
    }
}