<?php
/**
 * Class for collecting necessary information about news from database
 *
 * @author Andrew Voloshin <andrew.voloshin@oberig.com>
 *
 *
 */
class ApaLinesWidget extends BaseWidget
{
    /**
     * @var string comma separated names of the channels
     */
    public $channel;
    /**
     * @var string max age of news which are being visible starting from the newest
     */
    public $max_age_of_news;
    /**
     * @var string max count of news which are being visible starting from the newest
     */
    public $max_count_of_news;
    /**
     * @var string the data to return from widget
     */
    public $data;
    /**
     * Method collects overview info or detailed info about news line if id passed as $_GET['id']
     */
    public function init()
    {
        $module = Yii::app()->getModule('apalines');
        //Trim channels
        $this->channel = explode(',', $this->channel);
        $this->channel = array_map('trim', $this->channel);
        $cacheId = $this->getCacheId();

        if(false == ($this->data = Yii::app()->cache->get($cacheId))){
            $id = Yii::app()->request->getParam('id');
            if(!isset($id) || !$id){
                $this->setOverviewData();
            }else{
                $this->setDetailedData(Yii::app()->request->getParam('id'));
            }
            Yii::app()->cache->set($cacheId, $this->data, $module->cacheTime);
        }
    }
    /**
     * Method collects detailed info about news line with DOCID of filename which was passed as $_GET['id']
     *
     * @param string $id DOCID or filename of news line
     */
    private function setDetailedData($id)
    {
        $line = ApalinesNews::model()->find('docid = :id OR filename = :id', array(
            ':id' => $id
        ));

        if(isset($line)){
            $this->data = $this->lineToArray($line);
        }
    }
    /**
     * Method collects overview info about news in channels which were passed to widget
     */
    private function setOverviewData()
    {
        $this->data = array();

        $module = Yii::app()->getModule('apalines');
        $channelNames = $this->channel;
        //If should be shown one channel
        if (count($channelNames) == 1) {

            $channelName = $channelNames[0];
            if (isset($module->channels[$channelName])) {
                $channelSettings = $module->channels[$channelName];
            }
            if (isset($channelSettings)) {
                if (!isset($this->max_age_of_news)) {
                    $this->max_age_of_news = $channelSettings['max_age_of_news'];
                }
                if (!isset($this->max_count_of_news)) {
                    $this->max_count_of_news = $channelSettings['max_count_of_news'];
                }
            }
            if (!isset($this->max_age_of_news)) {
                $this->max_age_of_news = $channelSettings['max_age_of_news'];
            }
            if (!isset($this->max_count_of_news)) {
                $this->max_count_of_news = $channelSettings['max_count_of_news'];
            }

        } else {
            //If should be shown few channels
            $channelsCount = count($channelNames);

            if (!isset($this->max_age_of_news)) {
                $this->max_age_of_news = $module->total_max_age_of_news;
            }
            if (!isset($this->max_count_of_news)) {
                $this->max_count_of_news = $module->total_max_count_of_news;
            }
            //If news count is not divided into count of channels without remainder
            $modulo = $this->max_count_of_news % $channelsCount;
            $this->max_count_of_news = (integer)($this->max_count_of_news / $channelsCount);
        }

        //Go throw the channels
        foreach ($channelNames as $channelName){

            //equalize count by adding modulo to max count news of first channel
            $max_count_of_news = $this->max_count_of_news;
            if(count($channelNames) > 1 && isset($modulo)){
                $max_count_of_news += $modulo;
            }
            //Finding the news and adding them to $data
            $news = ApalinesNews::model()->findAll(array(
                'condition' => 'channel = :channel AND deleted <> 1 AND (FROM_UNIXTIME(ndate) > (NOW() - interval :max_age_of_news hour))',
                'order' => 'ndate DESC',
                'limit' => $max_count_of_news,
                'params' => array(
                    ':channel' => $channelName,
                    ':max_age_of_news' => $this->max_age_of_news
                )
            ));

            foreach ($news as $line) {
                if (empty($line->related_maindoc_id)) {
                    $this->data[] = $this->lineToArray($line);
                }
            }
        }
    }
    /**
     * Method migrates date from model of news line to array , it is used for news and background news
     * it also bind multimedia to news and background news
     *
     * @param object $line model of a news line or background news line
     * @return array - news or background line date
     */
    private function lineToArray($line)
    {
        $data = array();
        $bg_news = array();
        $multimedia = array();
        //main news line parameters
        $data['doc_id'] = $line->docid;
        $data['has_bg_news'] = $line->background_news;
        $data['title'] = $line->title;
        $data['place'] = $line->place;
        $data['source'] = $line->source;
        $data['header'] = $line->teaser;
        $data['text'] = $line->content;
        $data['author'] = $line->author;
        $data['keywords'] = $line->keywords;
        $data['filename'] = $line->filename;
        $data['channel'] = $line->channel;
        $data['date']  = $date = date('Y-m-d G:i:s',$line->ndate);
        $data['detailed_link'] = $_SERVER['REDIRECT_URL']."?id=".$line->filename;

        foreach($line->bg_news as $bg_line){
            $bg_news[] = $this->lineToArray($bg_line);
        }
        $data['bg_news'] = $bg_news;

        foreach($line->multimedia as $mm_line){
            $multimedia[] = $this->mm_lineToArray($mm_line);
        }
        $data['multimedia'] = $multimedia;

        return $data;
    }
    /**
     * Method migrates date from model of multimedia line to array , it is used for images and videos object
     * it also bind files (thumbnails, originals) which represents this multimedia
     *
     * @param object $line model of a multimedia line
     * @return array - multimedia line date
     */
    private function mm_lineToArray($mm_line){

        $data = array();

        $docid = $data['doc_id'] = $mm_line->docid;
        $customer_number = Yii::app()->getModule('apalines')->customer_number;
        $data['title'] = $mm_line->title;
        $data['place'] = $mm_line->place;
        $data['source'] = $mm_line->source;
        $data['author'] = $mm_line->author;
        $data['keywords'] = $mm_line->keywords;
        $data['filename'] = $mm_line->filename;
        $data['date']  = $date = date('Y-m-d G:i:s',$mm_line->ndate);

        $data['files'] = array();

        foreach($mm_line->files as $file){
            $fileData = array();

            $fileData['href']   = $this->controller->createAbsoluteUrl("/apalines/{$mm_line->channel}/images/{$file->href}");
            $fileData['width']  = $file->width;
            $fileData['height'] = $file->height;
            $fileData['format'] = $file->format;

            $type = '';
            switch ($file->size) {
                case 0:
                    $type = 'thumbnail';
                    break;
                case 1:
                    $type = 'original';
                    break;
                case 2:
                    $type = 'hires';
                    break;
            }
            if ($configSize = $this->getSize($type)) {
                $fileData['width']  = $configSize['width'];
                $fileData['height'] = $configSize['height'];
            }
            if (!isset($fileData['width'])
                || !isset($fileData['height'])) {
                $configSize = $this->getSize('default');
                $fileData['width']  = $configSize['width'];
                $fileData['height'] = $configSize['height'];
            }
            $data['files'][$type] = $fileData;
            unset($fileData);
        }

        switch($mm_line->type) {
            case 0:
                $data['type'] = 'image';
                break;
            case 1:
                $videoUrl    = Yii::app()->getModule('apalines')->video_url;
                $videoWidth  = 0;
                $videoHeight = 0;
                if($videoSize    = $this->getSize('video')){
                    $videoWidth  = $videoSize['width'];
                    $videoHeight = $videoSize['height'];
                }else{
                    $videoSize   = $this->getSize('default');
                    $videoWidth  = $videoSize['width'];
                    $videoHeight = $videoSize['height'];
                }
                $data['iframe'] =
                    "<iframe margin='0' padding='0' width='{$videoWidth}px' height='{$videoHeight}px'
                    src='{$videoUrl}?customer={$customer_number}&data={$docid}'
                    border='0' frameborder='0' >";

                $data['type'] = 'video';
                break;
        }

        return $data;
    }
    /**
     *
     * Method returns parsed size if it is specified in the config
     * @param string $type multimedia item type
     *
     * @return array - width and height
     */
    private function getSize($type) {

        $module    = Yii::app()->getModule('apalines');
        $type = ucfirst(strtolower($type)) . 'Size';
        $size = isset($module->$type)?$module->$type:'';

        $size = explode('x',$size);
        $size = array_map('trim', $size);
        if (count($size)==2) {
            return array(
                'width' => $size[0],
                'height'=> $size[1]
            );
        }
        return false;
    }
    /**
     * Method returns youngest timestamp among news lines, background news lines, multimedia lines
     * coupled with total count of news, background news, multimedia lines those channel names coincided with the passed channels
     *
     * @return string - youngest timestamp among news, background news and multimedia concatenated with their number
     */
    public function getTimestampAndCount(){

        $channels = "'".implode("','",$this->channel)."'";

        $timestamp = Yii::app()->db->createCommand("(SELECT `updated` FROM {{apalines_news}} WHERE `channel` IN ($channels) AND `deleted`<>1)
                                                    UNION
                                                    (SELECT `updated` FROM {{apalines_mm}} WHERE `channel` IN ($channels) AND `deleted`<>1)
                                                    ORDER BY `updated` DESC LIMIT 1
                                                    ")->queryAll();

        $timestamp = isset($timestamp[0]['updated'])?$timestamp[0]['updated']:false;
        $count = Yii::app()->db->createCommand("SELECT ((SELECT COUNT(*) FROM {{apalines_news}} WHERE `channel` IN ($channels))
                                                    +
                                                    (SELECT COUNT(*) FROM {{apalines_mm}} WHERE `channel` IN ($channels))) AS `count`
                                                    ")->queryAll();
        //To check whether items were removed
        $count = isset($count[0]['count'])?$count[0]['count']:false;

        $result = false;

        if($timestamp && $count){
            $result = "$timestamp $count";
        }

        return $result;
    }
    /**
     * Method returns timestamp of the news line which DOCID or filename was passed
     *
     * @param string $id DOCID or filename of the news file
     * @return string - timestamp
     */
    public function getItemTimestamp($id){

            $timestamp = Yii::app()->db->createCommand("(SELECT `updated` FROM {{apalines_news}} WHERE (`docid` = :id OR `filename` = :id) AND `deleted`<>1)
                                                        UNION
                                                        (SELECT `updated` FROM {{apalines_mm}} WHERE (`docid` = :id OR `filename` = :id) AND `deleted`<>1)
                                                        LIMIT 1
                                                        ")->bindValue(':id',$id)->queryAll();

            return isset($timestamp[0]['updated'])?$timestamp[0]['updated']:false;
    }
    /**
     * Method makes cache id for current widget content
     *
     * @return string - cache id for widget content
     */
    public function getCacheId(){
        $param = Yii::app()->request->getParam('id');
        $id = isset($param) && $param != false?$param:false;

        $cacheId = '';
        if($id){
            $cacheId = $this->getItemTimestamp($id);
            if(!$cacheId){
                return false;
            }
            $cacheId .= $id;
        }else{
            $cacheId = $this->getTimestampAndCount();
            if(!$cacheId){
                return false;
            }
            $cacheId .= $this->channel;
            $cacheId .= $this->max_age_of_news;
            $cacheId .= $this->max_count_of_news;
        }
        $cacheId = md5($cacheId);

        return "apa_lines_{$cacheId}";
    }
}